<?xml version="1.0" encoding="UTF-8"?>
<gui name="kubrick"
     version="10"
     xmlns="http://www.kde.org/standards/kxmlgui/1.0"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0
                         http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd">
<MenuBar>
    <Menu name="game">
	<Menu name="puzzles"> <text>&amp;Puzzle Type</text>
		<Action name="easy_list"/>
		<Action name="not_easy_list"/>
		<Action name="hard_list"/>
		<Action name="very_hard_list"/>
		<Action name="new_cube"/>
	</Menu>
    </Menu>
    <Menu name="move">
	<Action append="new_merge" name="game_restart" />
	<Action append="new_merge" name="redo_all" />
	<Action append="new_merge" name="standard_view" />
    </Menu>
    <Menu name="view">
	<Action append="new_merge" name="scene_1" />
	<Action append="new_merge" name="scene_2" />
	<Action append="new_merge" name="scene_3" />
    </Menu>
    <Menu name="demos"> <text>&amp;Demos</text>
	<Action name="move_demo"/>
	<Menu name="puzzles"> <text>&amp;Pretty Patterns</text>
		<ActionList name="patterns_list" />
	</Menu>
	<Menu name="puzzles"> <text>&amp;Solution Moves</text>
		<ActionList name="demo_moves_list"/>
	</Menu>
    </Menu>
    <Menu name="settings">
	<Action name="watch_shuffling"/>
	<Action name="watch_moves"/>
    </Menu>
</MenuBar>

<ToolBar name="mainToolBar" iconText="icononly" fullWidth="true">
    <Action name="scene_1" />
    <Action name="scene_2" />
    <Action name="scene_3" />
    <Separator />
    <Action name="game_new" />
    <Action name="game_save" />
    <Action name="game_load" />
    <Action name="game_restart" />
    <Separator />
    <Action name="move_undo" />
    <Action name="move_redo" />
    <Action name="move_solve" />
    <Action name="move_demo" />
    <Action name="standard_view" />
    <Separator />
    <Action name="singmaster_label" />
    <Action name="singmaster_moves" />
</ToolBar>

<ActionProperties>
    <Action shortcut="N; Ctrl+N"       name="game_new" />
    <Action shortcut="Q; Ctrl+Q"       name="game_quit" />
    <Action shortcut="Shift+U; F5"     name="game_restart" />
    <Action shortcut="Shift+R"         name="redo_all" />
    <Action shortcut="S"               name="move_solve" />
    <Action shortcut="Ctrl+D"          name="move_demo" />
</ActionProperties>

<StatusBar/>
</gui>
